%%%%%%%%%%%%%%%%%%%%%%%
% Homomorphic filtering
% Stphane BAZEILLE
% 15/10/07
%%%%%%%%%%

function result=homomorphic_filtering(I,type,RH,RL,cutoff)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% I = image (en niveau de gris et double entre 0 et 1)              %%
%% type= 'HighPassLiao' 'HighPassCufi' 'HighPassFilter'              %%
%%       'LowPassFilter' 'BandPassFilter' 'OnesFilter'               %%
%% !!!!!!!!! afficher avec imshow sans contrast_stretching !!!!!!!!! %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
     
[rows,cols,dim]=size(I); 

%figure;imshow(I);title('homo in');

% figure;hist(I,0.01:0.01:1.49);

disp(['Homomorphic filtering (',type,',',num2str(RH),',',num2str(RL),')']);

%Modlisation du filtre
if mod(cols,2) xrange = [-(cols-1)/2:(cols-1)/2]/(cols-1);
else       	   xrange = [-cols/2:(cols/2-1)]/cols;	
end
if mod(rows,2) yrange = [-(rows-1)/2:(rows-1)/2]/(rows-1);
else           yrange = [-rows/2:(rows/2-1)]/rows;	
end
[x,y] = meshgrid(xrange, yrange);
radius = sqrt(x.^2 + y.^2);
switch type
        case 'HighPassLiao' 
            DX=cols/cutoff;
            G = ones(rows,cols);
            for m = 1:rows
                for n = 1:cols
                    G (m,n) = ((RH-RL)*(1-exp(-(((m-rows/2))*((m-rows/2))+((n-cols/2))*((n-cols/2)))/(2*DX*DX))))+RL;
                    %G (m,n) = ((RH-RL)*(1-exp(-(((m-rows/8))*((m-rows/8))+((n-cols/8))*((n-cols/8)))/(2*DX*DX))))+RL;
                end
            end
        case 'HighPassCufi'
            cutoff = 0.4; % 0 - 0.5
            offset = 0.5;
            boost  = 2;
            scale  = 50;
            G = boost./(1.0 + exp(-scale*(radius-cutoff))) + offset;
        case 'HighPassFilter' 
            cutoff = 0.3;  % 0 - 0.5
            n      = 3;    % n >= 1
            boost  = 0.2;  % 0<= boost=<1
            G = (1-boost)*(1 - (1 ./ (1.0 + (radius ./ cutoff).^(2*n))))+boost;
        case 'LowPassFilter'
            cutoff = 0.1;  % 0 - 0.5
            n      = 3;    % n >= 1
            boost  = 0.2;  % 0<= boost <1 
            G = (1-boost) ./ (1.0 + (radius ./ cutoff).^(2*n))+boost;
        case 'BandPassFilter'
            cutoff = 0.1;  % 0 - 0.5
            cutin  = 0.01; % 0 - 0.5
            n = 3;         % n >= 1
            G = (1 ./ (1.0 + (radius ./ cutoff).^(2*n)))-(1 ./ (1.0 + (radius ./ cutin).^(2*n)));
        case 'OnesFilter'
            G=ones(rows,cols);
end

%Filtrage homomorphique
if(dim==1)
    Idft=fft2(log(I+0.01));
    filtree=G.*fftshift(Idft);
    result=exp(real(ifft2(ifftshift(filtree))));
else
    Idft=fft2(log(I(:,:,1)+0.01));
    filtree=G.*fftshift(Idft);
    result(:,:,1)=exp(real(ifft2(ifftshift(filtree))));

    Idft=fft2(log(I(:,:,2)+0.01));
    filtree=G.*fftshift(Idft);
    result(:,:,2)=exp(real(ifft2(ifftshift(filtree))));

    Idft=fft2(log(I(:,:,3)+0.01));
    filtree=G.*fftshift(Idft);
    result(:,:,3)=exp(real(ifft2(ifftshift(filtree))));
end

%figure;mesh(x,y,G);figure;
% figure;imshow(result);title('homo out ');
% figure;hist(result,0.01:0.01:1.49);
% result=suppress_outliers(result);
% result=contrast_stretching(result);
% figure;imshow(result);title('homo out with SO et CT');
% figure;hist(result,0.01:0.01:1.49);

end

%      %Degrader l'image
%      K=0.9;
%      Rc=10;
%      c=0.05;
%      i=0.001;
%      underwaternoise = (K*exp(-c*Rc*i*f));
%      Inoise= imnoise(I,'gaussian',0,0.001); 
%      I=real(ifft2(fft2(Inoise).*fftshift(underwaternoise)));
%      figure;imshow(I);     
     
%      %affichage du filtre
%      X=ones(rows,1)*[1:cols];Y=ones(cols,1)*[1:rows];Y=Y';figure;plot3(X,Y,G);grid on
%      %filtrage homomorphique de l'image par le filtre frequentiel genere
%      Idft=fft2(log(I+.01));
%      A=fftshift(abs(Idft));
%      imagesc(A);     
%      I=exp(real(ifft2(ifftshift(G.*fftshift(Idft)))));
%      %affichage du resultat
%      figure;imshow(I);title('Image Filtree');

%      OTF
%      K=0.9;c=0.05;Rc=1;
%      G=exp(c*Rc*radius)/K;
%      X=ones(rows,1)*[1:cols];Y=ones(cols,1)*[1:rows];Y=Y';figure;plot3(X,Y,G);grid on
